#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import math
import time
import copy
import numpy as np

from redcp import * # import data types

import matplotlib.pyplot as plt

import scipy.ndimage as ndimg
from sklearn.cluster import KMeans

import collections
import heapq

from queue import PriorityQueue

from bresenham import bresenham_line

class Planner:

    CELL_FREE = 0.4 ## < value of free cell
    CELL_MID = 0.5
    CELL_OCCU = 0.6 ## > value of occupied cell

    def __init__(self):
        pass

    def prep_mask(self, robot_size, resolution):
        mask = np.mgrid[
               -robot_size:robot_size + 1,
               -robot_size:robot_size + 1,
               ].reshape(2, -1)
        mask = mask[:, np.sqrt((mask[0]) ** 2 + (mask[1]) ** 2) < robot_size]
        return mask

    def grow_obstacles(self, grid_map: OccupancyGrid, robot_size) -> OccupancyGrid:
        """ Method to grow the obstacles to take into account the robot embodiment
        Args:
            grid_map: OccupancyGrid - gridmap for obstacle growing
            robot_size: float - size of the robot
        Returns:
            grid_map_grow: OccupancyGrid - gridmap with considered robot body embodiment
        """
        grid_map_grow = grid_map.clone()

        ## [t1c-plan] Implement obstacle growing 
        return grid_map_grow

    def successor(self, grid_map, state):
        ## TODO retrive successor nodes of the state used in A*
        children = None
        return children

    def euclidean(self, a, b):
        return np.sqrt((a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2)

    def heuristic(self, a, b):
        return self.euclidean(a, b)

    def plan_path(self, grid_map: OccupancyGrid, start: Pose, goal: Pose) -> Path:
        """ Method to plan the path from start to the goal pose on the grid
        Args:
            grid_map: OccupancyGrid - grid map for planning
            start: Pose - robot start pose
            goal: Pose - robot goal pose
        Returns:
            path: Path - path between the start and goal Pose on the map
        """

        path = Path()

        ## [t1c-plan] Implement grid-based path planning such as A*

        return path

    def simplify_path(self, grid_map: OccupancyGrid, path: Path) -> Path: 
        """ Method to simplify the found path on the grid
        Args:
            grid_map: OccupancyGrid - gridmap for obstacle growing
            path: Path - path to be simplified
        Returns:
            path_simple: Path - simplified path
        """
        if grid_map is None or path is None:
            return None

        ## [t1c-plan] Implement path simplifiction using ray-casting based on bresenham algorithm

        path_simplified = Path()

        return path_simplified

