
def bresenham_line(start: (int, int), goal: (int, int)):
    """Bresenham's line algorithm
    Args:
        start: (int, int) - start coordinate
        goal: (int, int) - goal coordinate
    Returns:
        interlying points between the start and goal coordinate
    """
    (x0, y0) = start
    (x1, y1) = goal
    line = []
    dx = abs(x1 - x0)
    dy = abs(y1 - y0)
    x, y = x0, y0
    sx = -1 if x0 > x1 else 1
    sy = -1 if y0 > y1 else 1
    if dx > dy:
        err = dx / 2.0
        while x != x1:
            line.append((x,y))
            err -= dy
            if err < 0:
                y += sy
                err += dx
            x += sx
    else:
        err = dy / 2.0
        while y != y1:
            line.append((x,y))
            err -= dx
            if err < 0:
                x += sx
                err += dy
            y += sy
    x = goal[0]
    y = goal[1]
    return line

